%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%                     The evaluation code for 
%%% SHREC'12 -- Sketch-Based 3D Shape Retrieval   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Input:
%       rank list files  -- R00001 ~ R00250

%Output:
%       Precision Recall performance                         -- "PR_e.txt"
%       Other performance metrics (each model and average)   -- "Stats_e.txt"

%Evaluation measures:
%       NN, 1-Tier, 2-Tier, e_Measure, DCG

%Author: Bo Li
%Email: libo0002@ntu.edu.sg
%Date: February 24, 2012
%@NIST, Gaithersburg, US

%Please cite:
% SHREC'12 -- Sketch-Based 3D Shape Retrieval, 3DOR'12, 2012



clear all;
clc

number_of_queries=250;
querynum=[21,19,17,20,19,19,19,19,19,19,20,19,20];
queryclassname={'human','cup','glasses','plane','ant','chair','octopus','table','teddy','hand','plier','fish','bird'};
querysum=cumsum(querynum);
for qqq=1:number_of_queries
    if (qqq<10)
        query_labels{qqq,1}=strcat('R0000',num2str(qqq));
    elseif (qqq<100)
        query_labels{qqq,1}=strcat('R000',num2str(qqq));  
    else
        query_labels{qqq,1}=strcat('R00',num2str(qqq));
    end
    
    if (qqq<=querysum(1,1))
        query_labels{qqq,2}=1;
    else
        for i=2:13
            if ((qqq>querysum(1,i-1)) & (qqq<=querysum(1,i)))
               break;
            end
        end
        query_labels{qqq,2}=i;
    end
end;

number_of_target=400;
for qqq=1:number_of_target     
    if (qqq<10)
        target_labels{qqq,1}=strcat('D0000',num2str(qqq));
    elseif (qqq<100)
        target_labels{qqq,1}=strcat('D000',num2str(qqq));  
    else
        target_labels{qqq,1}=strcat('D00',num2str(qqq));
    end
    target_labels{qqq,2}=ceil(qqq/20); 
end;

C=20;

%The folder that contains the rank list files
RANK_DIR='C:\Projects\SHREC 2012\SBR\Generic3D\Attendee\Evaluation\';

P_points=zeros(number_of_queries,C);
Av_Precision=zeros(1,number_of_queries); %not considering recall=0
NN=zeros(1,number_of_queries);
FT=zeros(1,number_of_queries);
ST=zeros(1,number_of_queries);
dcg=zeros(1,number_of_queries);
E=zeros(1,number_of_queries);
filename=[RANK_DIR 'Stats_e.txt'];
fid=fopen(filename,'w');
fclose(fid); 
for qqq=1:number_of_queries;
    
    query_name=query_labels{qqq,1};
    filename=[RANK_DIR query_name];
    
    [R,S,q]=read_rank_list_shrec12_extended(filename);

    query_class=query_labels{qqq,2};
    
    retrieved_class=zeros(1,length(R));
    for r=1:length(R);
        for i=1:number_of_target
            if (strcmp(R{r},target_labels(i,1))==1)
                n=i;
                break;
            end
        end
        retrieved_class(r)=target_labels{n,2};
    end;
    
   
    G=query_class==retrieved_class;
    G_sum=cumsum(G); 
    
    R_points=zeros(1,C);
    for rec=1:C
        R_points(rec)=find((G_sum==rec),1);
    end;
    
    P_points(qqq,:)=G_sum(R_points)./R_points;
    Av_Precision(qqq)=mean(P_points(qqq,:));
    
    NN(qqq)=G(1);
    FT(qqq)=G_sum(C)/C;
    ST(qqq)=G_sum(2*C)/C;
    P_32=G_sum(32)/32;
    R_32=G_sum(32)/C;
    
    if (P_32==0)&&(R_32==0);
        E(qqq)=0;
    else
        E(qqq)=2*P_32*R_32/(P_32+R_32);
    end;
    
    NORM_VALUE=1+sum(1./log2([2:C]));
    dcg_i=(1./log2([2:length(R)])).*G(2:end);
    dcg_i=[G(1);dcg_i(:)];
    dcg(qqq)=sum(dcg_i)/NORM_VALUE;
    
    filename=[RANK_DIR 'Stats_e.txt'];
    fid=fopen(filename,'a');
    fprintf(fid,'No.%d: %2.3f\t %2.3f\t %2.3f\t %2.3f\t %2.3f\n',qqq, NN(qqq),FT(qqq),ST(qqq),E(qqq),dcg(qqq));
    fclose(fid);    
end;

NN_av=mean(NN)
FT_av=mean(FT)
ST_av=mean(ST)
dcg_av=mean(dcg)
E_av=mean(E)
Mean_Av_Precision=mean(Av_Precision) %not considering recall=0
filename=[RANK_DIR 'Stats_e.txt'];
fid=fopen(filename,'a');
fprintf(fid,'       NN      FT          ST          E          DCG \n');
fprintf(fid,'%2.3f\t %2.3f\t %2.3f\t %2.3f\t %2.3f\n',NN_av,FT_av,ST_av,E_av,dcg_av);
fclose(fid);


Precision=mean(P_points);
Recall=[1:C]/C;

plot(Recall,Precision);
hold on;
plot(Recall,Precision,'*');
axis([0 1 0 1])
hold off;

filename=[RANK_DIR 'PR_e.txt'];
fid=fopen(filename,'w');
for c=1:C
    fprintf(fid,'%2.3f\t %2.3f\n',Recall(c),Precision(c));
end;
fclose(fid);



